unit Mathgraphunit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Image1: TImage;
    BtFunc1: TButton;
    BtFunc2: TButton;
    BtFunc3: TButton;
    BtReCalc: TButton;
    BtZoomIn: TButton;
    BtZoomOut: TButton;
    BtAspect: TButton;
    BtExit: TButton;
    LbTitle: TLabel;
    LbMsg: TLabel;
    GroupBox1: TGroupBox;
    LbEditA: TLabeledEdit;
    LbEditB: TLabeledEdit;
    LbEditC: TLabeledEdit;
    LbEditD: TLabeledEdit;
    GroupBox2: TGroupBox;
    LbEditXmin: TLabeledEdit;
    LbEditXmax: TLabeledEdit;
    LbEditYmin: TLabeledEdit;
    LbEditYmax: TLabeledEdit;
    procedure FormCreate(Sender: TObject);
    procedure BtExitClick(Sender: TObject);
    procedure BtFunc1Click(Sender: TObject);
    procedure BtFunc2Click(Sender: TObject);
    procedure BtFunc3Click(Sender: TObject);
    procedure BtReCalcClick(Sender: TObject);
    procedure BtZoomInClick(Sender: TObject);
    procedure BtZoomOutClick(Sender: TObject);
    procedure BtAspectClick(Sender: TObject);
  private
    procedure SetGrWin;
    procedure ClearScreen;
    procedure ReadCoef;
    procedure ReadScale;
    procedure DrawGraph;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  Xtransl, Ytransl, Xref, Yref: real;
  Xmin, Xmax, Ymin, Ymax: real;
  CoefA, CoefB, CoefC, CoefD: real;
  FuncNo: integer;
  Err: Boolean;

implementation

{$R *.dfm}

procedure TForm1.SetGrWin;
begin
  {calculate global translation factors}
  Xtransl := Image1.ClientWidth/(Xmax - Xmin);
  Ytransl := -Image1.ClientHeight/(Ymax - Ymin);
  Xref := Xmin;
  Yref := Ymax;
end;

function Xim(X: real): integer;
var Xtemp: real;
begin
  {convert X-value in horizontal window coordinate}
  Xtemp := Xtransl * (X - Xref);
  if Xtemp > Maxint then Xtemp := Maxint;
  if Xtemp < -Maxint then Xtemp := -Maxint;
  Xim := round(Xtemp);
end;

function Yim(Y: real): integer;
var Ytemp: real;
begin
  {convert Y-value in vertical window coordinate}
  Ytemp := Ytransl * (Y - Yref);
  if Ytemp > Maxint then Ytemp := Maxint;
  if Ytemp < -Maxint then Ytemp := -Maxint;
  Yim := round(Ytemp);
end;

procedure TForm1.ClearScreen;
var I: integer;
begin
  with Image1.Canvas do begin
    {clearscreen}
    Pen.Color := clWhite;
    for I := 0 to ClientHeight do begin
      MoveTo(0,I);
      LineTo(ClientWidth,I);
      end;
    {display coordinate origin if zero in window}
    Pen.Color := clGreen;
    if ((Xmin <=0 ) and (Xmax >= 0)) then begin
      MoveTo(Xim(0),0);
      LineTo(Xim(0),ClientHeight);
      end;
    if ((Ymin <=0 ) and (Ymax >= 0)) then begin
      MoveTo(0,Yim(0));
      LineTo(ClientWidth,Yim(0));
      end;
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  {initialize window}
  Xmin := -100; Xmax:= 100;
  Ymin := -50; Ymax:= 50;
  SetGrWin;
  ClearScreen;
  FuncNo := 0;
  {prevent user from typing in values when no function selected}
  GroupBox1.Enabled := False;
  GroupBox2.Enabled := False;
  Err := False;
  DecimalSeparator := '.';
end;

procedure TForm1.BtExitClick(Sender: TObject);
begin
  Close;
end;

function CalcY(FuncNo: integer; X:real):real;
begin
  {calculate Y for any value of X}
  case FuncNo of
  1: CalcY := CoefA * X * X * X + CoefB * X * X  + CoefC * X + CoefD;
  2: CalcY := CoefA * Sin(CoefB * X + CoefC) + CoefD;
  3: CalcY := CoefA * exp(sqr(X + CoefC)/CoefB) + CoefD;
  else CalcY := 0; end;
end;

function ShowFunc(FuncNo: integer):string;
var FuncStr, StrA, StrB, StrC, StrD: string;
  {for positive values in formulae a + is necessary}
  function CoefSign(X: real):string;
  begin
    if X>=0 then CoefSign := '+' else CoefSign := '';
  end;
begin
  {present formula in readable form}
  FuncStr := 'Function  Y = ';
  StrA := FloatToStrF(CoefA, ffGeneral, 9, 3);
  StrB := FloatToStrF(CoefB, ffGeneral, 9, 3);
  StrC := FloatToStrF(CoefC, ffGeneral, 9, 3);
  StrD := FloatToStrF(CoefD, ffGeneral, 9, 3);
  case FuncNo of
  1: begin
     if CoefA<>0 then FuncStr := FuncStr + StrA + 'X^3';
     if CoefB<>0 then FuncStr := FuncStr + CoefSign(CoefB) + StrB + 'X^2';
     if CoefC<>0 then FuncStr := FuncStr + CoefSign(CoefC) + StrC + 'X';
     if CoefD<>0 then FuncStr := FuncStr + CoefSign(CoefD) + StrD;
     end;
  2: begin
     FuncStr := FuncStr + StrA + '*Sin(';
     FuncStr := FuncStr + StrB + 'X';
     if CoefC<>0 then FuncStr := FuncStr + CoefSign(CoefC) + StrC;
     FuncStr := FuncStr + ')';
     if CoefD<>0 then FuncStr := FuncStr + CoefSign(CoefD) + StrD;
     end;
  3: begin
     FuncStr := FuncStr + StrA + '*e^[(X';
     if CoefC<>0 then FuncStr := FuncStr + CoefSign(CoefC) + StrC;
     FuncStr := FuncStr + ')^2]/(' + StrB + ')';
     if CoefD<>0 then FuncStr := FuncStr + CoefSign(CoefD) + StrD;
     end;
  end;
  ShowFunc := FuncStr;
end;

procedure TForm1.ReadCoef;
var Ft:Integer;
begin
  {read coefficients from editboxes and check validity}
  val(LbEditA.Text,CoefA,Ft);
  if Ft>0 then begin
    Err := True;
    LbEditA.Font.color := clRed;
    end else LbEditA.Font.color := clBlack;
  val(LbEditB.Text,CoefB,Ft);
  if ((Ft>0) or ((FuncNo=3) and (CoefB=0))) then begin
    Err := True;
    LbEditB.Font.color := clRed;
    end else LbEditB.Font.color := clBlack;
  val(LbEditC.Text,CoefC,Ft);
  if Ft>0 then begin
    Err := True;
    LbEditC.Font.color := clRed;
    end else LbEditC.Font.color := clBlack;
  val(LbEditD.Text,CoefD,Ft);
  if Ft>0 then begin
    Err := True;
    LbEditD.Font.color := clRed;
    end else LbEditD.Font.color := clBlack;
end;

procedure TForm1.ReadScale;
var Ft:Integer; Temp: real; Changed: Boolean;
begin
  {read border values from editboxes and check validity}
  val(LbEditXmin.Text,Xmin,Ft);
  if Ft>0 then begin
    Err := True;
    LbEditXmin.Font.color := clRed;
    end else LbEditXmin.Font.color := clBlack;
  val(LbEditXmax.Text,Xmax,Ft);
  if Ft>0 then begin
    Err := True;
    LbEditXmax.Font.color := clRed;
    end else LbEditXmax.Font.color := clBlack;
  val(LbEditYmin.Text,Ymin,Ft);
  if Ft>0 then begin
    Err := True;
    LbEditYmin.Font.color := clRed;
    end else LbEditYmin.Font.color := clBlack;
  val(LbEditYmax.Text,Ymax,Ft);
  if Ft>0 then begin
    Err := True;
    LbEditYmin.Font.color := clRed;
    end else LbEditYmax.Font.color := clBlack;
  {check if parameters are fool-proof}
  Changed := False;
  if Xmax < Xmin then begin
    Temp := Xmin; Xmin := Xmax; Xmax := Temp;
    Changed := true; end;
  if Xmax = Xmin then begin
    Xmax := Xmax + 0.1;
    Changed := true; end;
  if Ymax < Ymin then begin
    Temp := Ymin; Ymin := Ymax; Ymax := Temp;
    Changed := true; end;
  if Ymax = Ymin then begin Ymax := Ymax + 0.1;
    Changed := true; end;
  if Changed then begin
    LbEditXmin.Text := FloatToStrF(Xmin, ffGeneral, 9, 3);
    LbEditXmax.Text := FloatToStrF(Xmax, ffGeneral, 9, 3);
    LbEditYmin.Text := FloatToStrF(Ymin, ffGeneral, 9, 3);
    LbEditYmax.Text := FloatToStrF(Ymax, ffGeneral, 9, 3);
    end;
end;

procedure TForm1.BtFunc1Click(Sender: TObject);
begin
  {initial values function 1:	Y = AX + BX + CX + D}
  FuncNo := 1;
  GroupBox1.Enabled := True;
  GroupBox2.Enabled := True;
  LbEditA.Text := '-6';
  LbEditB.Text := '5';
  LbEditC.Text := '4';
  LbEditD.Text := '3';
  LbEditXmin.Text := '-2';
  LbEditXmax.Text := '3';
  LbEditYmin.Text := '-4';
  LbEditYmax.Text := '12';
  BtReCalcClick(Sender);
end;

procedure TForm1.BtFunc2Click(Sender: TObject);
begin
  {initial values function 2:	Y = A*(sin(BX + C) +D}
  FuncNo := 2;
  GroupBox1.Enabled := True;
  GroupBox2.Enabled := True;
  LbEditA.Text := '6';
  LbEditB.Text := '5';
  LbEditC.Text := '4';
  LbEditD.Text := '3';
  LbEditXmin.Text := '-2';
  LbEditXmax.Text := '2';
  LbEditYmin.Text := '-5';
  LbEditYmax.Text := '11';
  BtReCalcClick(Sender);
end;

procedure TForm1.BtFunc3Click(Sender: TObject);
begin
  {initial values function 3:	Y = A*e^[((X + C))/B] + D}
  FuncNo := 3;
  GroupBox1.Enabled := True;
  GroupBox2.Enabled := True;
  LbEditA.Text := '100';
  LbEditB.Text := '-40';
  LbEditC.Text := '-10';
  LbEditD.Text := '0';
  LbEditXmin.Text := '-10';
  LbEditXmax.Text := '30';
  LbEditYmin.Text := '-5';
  LbEditYmax.Text := '110';
  BtReCalcClick(Sender);
end;

procedure TForm1.BtReCalcClick(Sender: TObject);
begin
  {check if input is valid}
  LbMsg.Font.Color := clRed;
  if FuncNo=0 then begin
    LbMsg.Caption := 'No Function selected';
    Exit; end;
  ReadCoef;
  ReadScale;
  if Err=True then begin
    LbMsg.Caption := 'Error in input';
    Err := False;
    Exit; end;
  if ((CoefA=0) and (CoefB=0) and (CoefC=0) and (CoefD=0)) then begin
    LbMsg.Caption := 'All coefficients are zero';
    Exit; end;
  {calculate graph of chosen function}
  LbMsg.Font.Color := clBlack;
  LbMsg.Caption := Showfunc(FuncNo);
  SetGrWin;
  ClearScreen;
  DrawGraph;
end;

procedure TForm1.DrawGraph;
var XStep, X, Y: real;
begin
with Image1.Canvas do begin
  Pen.Color := clBlack;
  {set initial position and determine step-size}
  X := Xmin;
  Y := CalcY(FuncNo, X);
  XStep := (Xmax - Xmin)/ClientWidth;
  MoveTo(Xim(X), Yim(Y));
  {draw the graph}
  repeat
    X := X + XStep;
    Y := CalcY(FuncNo, X);
    LineTo(Xim(X), Yim(Y));
  until X >= Xmax;
  end;
end;

procedure TForm1.BtZoomInClick(Sender: TObject);
var Temp, Mid:real;
begin
  {determine center of graphic window and halve size}
  Mid := (Xmax + Xmin)/2;
  Temp :=(Xmax - Xmin)/4;
  Xmin := Mid - Temp;
  Xmax := Mid + Temp;
  Mid := (Ymax + Ymin)/2;
  Temp :=(Ymax - Ymin)/4;
  Ymin := Mid - Temp;
  Ymax := Mid + Temp;
  LbEditXmin.Text := FloatToStrF(Xmin, ffGeneral, 9, 3);
  LbEditXmax.Text := FloatToStrF(Xmax, ffGeneral, 9, 3);
  LbEditYmin.Text := FloatToStrF(Ymin, ffGeneral, 9, 3);
  LbEditYmax.Text := FloatToStrF(Ymax, ffGeneral, 9, 3);
  BtReCalcClick(Sender);
end;

procedure TForm1.BtZoomOutClick(Sender: TObject);
var Temp, Mid:real;
begin
  {determine center of graphic window and double size}
  Mid := (Xmax + Xmin)/2;
  Temp :=(Xmax - Xmin);
  Xmin := Mid - Temp;
  Xmax := Mid + Temp;
  Mid := (Ymax + Ymin)/2;
  Temp :=(Ymax - Ymin);
  Ymin := Mid - Temp;
  Ymax := Mid + Temp;
  LbEditXmin.Text := FloatToStrF(Xmin, ffGeneral, 9, 3);
  LbEditXmax.Text := FloatToStrF(Xmax, ffGeneral, 9, 3);
  LbEditYmin.Text := FloatToStrF(Ymin, ffGeneral, 9, 3);
  LbEditYmax.Text := FloatToStrF(Ymax, ffGeneral, 9, 3);
  BtReCalcClick(Sender);
end;

procedure TForm1.BtAspectClick(Sender: TObject);
var Temp, Mid:real;begin
  {determine center of graphic window and restore aspect ratio}
  Mid := (Xmax + Xmin)/2;
  Temp := (Image1.ClientWidth/Image1.ClientHeight) * (Ymax - Ymin)/2;
  Xmin := Mid - Temp;
  Xmax := Mid + Temp;
  LbEditXmin.Text := FloatToStrF(Xmin, ffGeneral, 9, 3);
  LbEditXmax.Text := FloatToStrF(Xmax, ffGeneral, 9, 3);
  BtReCalcClick(Sender);
end;

end.
